@@ -55,7 +55,8 @@ CCPCore;
 #define CCP_CORE(c)		     \
     CCPCore *cc = GET_CCP_CORE (c)
 
-#define CCP_UPDATE_TIMEOUT 250
+#define CCP_UPDATE_MIN_TIMEOUT 250
+#define CCP_UPDATE_MAX_TIMEOUT 4000
 #define CORE_VTABLE_NAME  "core"
 
 static void
@@ -146,6 +147,50 @@ ccpSetValueToValue (CompObject      *object,
     }
 }
 
+static CompOptionType
+ccpCCSTypeToCompizType (CCSSettingType st, CompOptionType *ct)
+{
+    switch (st) {
+    case TypeBool:
+	*ct = CompOptionTypeBool;
+	break;
+    case TypeInt:
+	*ct = CompOptionTypeInt;
+	break;
+    case TypeFloat:
+	*ct = CompOptionTypeFloat;
+	break;
+    case TypeColor:
+	*ct = CompOptionTypeColor;
+	break;
+    case TypeString:
+	*ct = CompOptionTypeString;
+	break;
+    case TypeMatch:
+	*ct = CompOptionTypeMatch;
+	break;
+    case TypeKey:
+	*ct = CompOptionTypeKey;
+	break;
+    case TypeButton:
+	*ct = CompOptionTypeButton;
+	break;
+    case TypeEdge:
+	*ct = CompOptionTypeEdge;
+	break;
+    case TypeBell:
+	*ct = CompOptionTypeBell;
+	break;
+    case TypeList:
+	*ct = CompOptionTypeList;
+	break;
+    default:
+	return FALSE;
+    }
+
+    return TRUE;
+}
+
 static void
 ccpConvertPluginList (CCSSetting          *s,
 		      CCSSettingValueList list,
@@ -197,6 +242,9 @@ ccpSettingToValue (CompObject      *object,
 
 	ccsGetList (s, &list);
 
+	if (!ccpCCSTypeToCompizType (s->info.forList.listType, &v->list.type))
+	    v->list.type = CompOptionTypeBool;
+
 	if ((strcmp (s->name, "active_plugins") == 0) &&
 	    (strcmp (s->parent->name, CORE_VTABLE_NAME) == 0))
 	{
@@ -345,47 +393,23 @@ ccpValueToSetting (CompObject      *object,
     ccsFreeSettingValue (value);
 }
 
-static Bool
-ccpSameType (CCSSettingType st, CompOptionType ot)
-{
-    if (st == TypeBool && ot == CompOptionTypeBool)
-	return TRUE;
-    if (st == TypeInt && ot == CompOptionTypeInt)
-	return TRUE;
-    if (st == TypeFloat && ot == CompOptionTypeFloat)
-	return TRUE;
-    if (st == TypeColor && ot == CompOptionTypeColor)
-	return TRUE;
-    if (st == TypeString && ot == CompOptionTypeString)
-	return TRUE;
-    if (st == TypeMatch && ot == CompOptionTypeMatch)
-	return TRUE;
-    if (st == TypeKey && ot == CompOptionTypeKey)
-	return TRUE;
-    if (st == TypeButton && ot == CompOptionTypeButton)
-	return TRUE;
-    if (st == TypeEdge && ot == CompOptionTypeEdge)
-	return TRUE;
-    if (st == TypeBell && ot == CompOptionTypeBell)
-	return TRUE;
-    if (st == TypeList && ot == CompOptionTypeList)
-	return TRUE;
-    return FALSE;
-}
-
 static Bool
 ccpTypeCheck (CCSSetting *s, CompOption *o)
 {
+    CompOptionType ot;
+
     switch (s->type)
     {
     case TypeList:
-	return ccpSameType (s->type, o->type) &&
-	       ccpSameType (s->info.forList.listType, o->value.list.type);
+	return ccpCCSTypeToCompizType (s->type, &ot) && (ot == o->type) &&
+	       ccpCCSTypeToCompizType (s->info.forList.listType, &ot) &&
+	       (ot == o->value.list.type);
 	break;
     default:
-	return ccpSameType (s->type, o->type);
+	return ccpCCSTypeToCompizType (s->type, &ot) && (ot == o->type);
 	break;
     }
+
     return FALSE;
 }
 
@@ -429,18 +453,14 @@ ccpSetOptionFromContext (CompObject *object,
     if (!ccpTypeCheck (setting, o))
 	return;
 
-    
     compInitOptionValue (&value);
-    
     ccpSettingToValue (object, setting, &value);
 
     cc->applyingSettings = TRUE;
-
     (*core.setOptionForPlugin) (object, plugin, o->name, &value);
+    cc->applyingSettings = FALSE;
 
     compFiniOptionValue (&value, o->type);
-
-    cc->applyingSettings = FALSE;
 }
 
 static void
@@ -722,8 +742,9 @@ ccpInitCore (CompPlugin *p,
 
     cc->applyingSettings = FALSE;
 
-    cc->reloadHandle = compAddTimeout (0, ccpReload, 0);
-    cc->timeoutHandle = compAddTimeout (CCP_UPDATE_TIMEOUT,
+    cc->reloadHandle = compAddTimeout (0, 0, ccpReload, 0);
+    cc->timeoutHandle = compAddTimeout (CCP_UPDATE_MIN_TIMEOUT,
+					CCP_UPDATE_MAX_TIMEOUT,
 					ccpTimeout, 0);
 
     core.base.privates[corePrivateIndex].ptr = cc;
