@@ -53,7 +53,7 @@ extern int xmlLoadExtDtdDefaultValue;
 
 Bool usingProtobuf = TRUE;
 
-#define PB_ABI_VERSION 20081004
+#define PB_ABI_VERSION 20090314
 
 typedef metadata::PluginInfo PluginInfoMetadata;
 typedef metadata::PluginBrief PluginBriefMetadata;
@@ -85,11 +85,11 @@ getLocale ()
     if (!lang || !strlen (lang))
 	lang = getenv ("LC_MESSAGES");
 
-    return lang;
+    return lang ? lang : (char *)"";
 }
 
 std::string curLocale = std::string (getLocale ());
-std::string shortLocale = curLocale.find ('.') < 0 ?
+std::string shortLocale = curLocale.find ('.') == std::string::npos ?
     curLocale : curLocale.substr (0, curLocale.find ('.'));
 
 #endif
@@ -281,8 +281,6 @@ initEdgeValuePB (CCSSettingValue * v,
 		 CCSSettingInfo * i,
 		 const GenericValueMetadata & value)
 {
-    int k, num;
-
     v->value.asEdge = 0;
 
     if (value.has_edge_value ())
@@ -516,7 +514,6 @@ addOptionForPluginPB (CCSPlugin * plugin,
 		      const StringList & subgroups,
 		      const OptionMetadata & option)
 {
-    int num = 0;
     CCSSetting *setting;
 
     if (ccsFindSetting (plugin, name, isScreen, screen))
@@ -677,7 +674,6 @@ addOptionFromPB (CCSPlugin * plugin,
 {
     const char *name;
     Bool readonly = FALSE;
-    int i;
 
     name = option.name ().c_str ();
 
@@ -688,7 +684,7 @@ addOptionFromPB (CCSPlugin * plugin,
 
     if (isScreen)
     {
-	for (i = 0; i < plugin->context->numScreens; i++)
+	for (unsigned i = 0; i < plugin->context->numScreens; i++)
 	    addOptionForPluginPB (plugin, name, TRUE,
 				  plugin->context->screens[i],
 				  groups, subgroups, option);
@@ -1009,9 +1005,7 @@ getOptionType (const char *name)
 	{ "match", TypeMatch },
 	{ "list", TypeList }
     };
-    int i;
-
-    for (i = 0; i < sizeof (map) / sizeof (map[0]); i++)
+    for (unsigned i = 0; i < sizeof (map) / sizeof (map[0]); i++)
 	if (strcasecmp (name, map[i].name) == 0)
 	    return map[i].type;
 
@@ -1439,7 +1433,7 @@ initEdgeValue (CCSSettingValue * v,
 {
     xmlNode **nodes;
     char *value;
-    int j, k, num;
+    int k, num;
 
     v->value.asEdge = 0;
 
@@ -1461,7 +1455,7 @@ initEdgeValue (CCSSettingValue * v,
 	value = getStringFromXPath (node->doc, nodes[k], "@name");
 	if (value)
 	{
-	    for (j = 0; j < sizeof (edge) / sizeof (edge[0]); j++)
+	    for (unsigned j = 0; j < sizeof (edge) / sizeof (edge[0]); j++)
 	    {
 		if (strcasecmp ((char *) value, edge[j]) == 0)
 		    v->value.asEdge |= (1 << j);
@@ -2170,8 +2164,6 @@ addOptionFromXMLNode (CCSPlugin * plugin,
     char *type;
     char *readonly;
     Bool isReadonly;
-    Bool screen;
-    int i;
 
     if (!node)
 	return;
@@ -2201,7 +2193,7 @@ addOptionFromXMLNode (CCSPlugin * plugin,
 
     if (isScreen)
     {
-	for (i = 0; i < plugin->context->numScreens; i++)
+	for (unsigned i = 0; i < plugin->context->numScreens; i++)
 	    addOptionForPlugin (plugin, name, type, isReadonly, TRUE,
 				plugin->context->screens[i], node,
 				groupListPBv, subgroupListPBv, optionPBv);
@@ -2658,20 +2650,18 @@ checkAndLoadProtoBuf (char *pbPath,
 		      struct stat *xmlStat,
 		      PluginBriefMetadata *pluginBriefPB)
 {
-    Bool needsUpdate = FALSE;
     const PluginInfoMetadata &pluginInfoPB = pluginBriefPB->info ();
 
     if (pbStat->st_mtime < xmlStat->st_mtime ||     // is .pb older than .xml?
 	!loadPluginMetadataFromProtoBuf (pbPath, pluginBriefPB, NULL) ||
 	(!basicMetadata && pluginBriefPB->info ().basic_metadata ()) ||
 	pluginInfoPB.pb_abi_version () != PB_ABI_VERSION ||
+	pluginInfoPB.time () != (unsigned long)xmlStat->st_mtime ||
+	// xml modification time mismatch?
 	(pluginInfoPB.locale () != "NONE" &&
 	 pluginInfoPB.locale () != shortLocale))
     {
 	// .pb needs update
-
-	remove (pbPath); // Attempt to remove .pb
-
 	return FALSE;
     }
     return TRUE;
@@ -2681,7 +2671,8 @@ checkAndLoadProtoBuf (char *pbPath,
 static void
 writePBFile (char *pbFilePath,
 	     PluginMetadata *pluginPB,
-	     PluginBriefMetadata *pluginBriefPB)
+	     PluginBriefMetadata *pluginBriefPB,
+	     struct stat *xmlStat)
 {
     if (!createProtoBufCacheDir ())
 	return;
@@ -2698,6 +2689,7 @@ writePBFile (char *pbFilePath,
 	pluginInfoPB = pluginBriefPB->mutable_info ();
 	pluginInfoPB->set_pb_abi_version (PB_ABI_VERSION);
 	pluginInfoPB->set_locale (shortLocale);
+	pluginInfoPB->set_time ((unsigned long)xmlStat->st_mtime);
 	pluginInfoPB->set_brief_metadata (TRUE);
     }
 
@@ -2779,11 +2771,11 @@ loadPluginFromXMLFile (CCSContext * context, char *xmlName, char *xmlDirPath)
 
 #ifdef USE_PROTOBUF
     char *name = NULL;
+    struct stat xmlStat;
+    Bool removePB = FALSE;
 
     if (usingProtobuf)
     {
-	struct stat xmlStat;
-
 	if (stat (xmlFilePath, &xmlStat))
 	{
 	    free (xmlFilePath);
@@ -2792,7 +2784,6 @@ loadPluginFromXMLFile (CCSContext * context, char *xmlName, char *xmlDirPath)
 
 	// Check if the corresponding .pb exists in cache
 	Bool error = TRUE;
-	int lenXMLName = strlen (xmlName);
 	struct stat pbStat;
 
 	name = strndup (xmlName, strlen (xmlName) - 4);
@@ -2817,26 +2808,32 @@ loadPluginFromXMLFile (CCSContext * context, char *xmlName, char *xmlDirPath)
 	    error = stat (pbFilePath, &pbStat);
 	}
 
-	if (!error &&
-	    checkAndLoadProtoBuf (pbFilePath, &pbStat, &xmlStat,
-				  &persistentPluginBriefPB))
+	if (!error)
 	{
-	    // Found and loaded .pb
-	    if (!strcmp (name, "core"))
-		addCoreSettingsFromPB (context, persistentPluginBriefPB.info (),
-				       pbFilePath, xmlFilePath);
+	    if (checkAndLoadProtoBuf (pbFilePath, &pbStat, &xmlStat,
+				      &persistentPluginBriefPB))
+	    {
+		// Found and loaded .pb
+		if (!strcmp (name, "core"))
+		    addCoreSettingsFromPB (context,
+					   persistentPluginBriefPB.info (),
+					   pbFilePath, xmlFilePath);
+		else
+		    addPluginFromPB (context, persistentPluginBriefPB.info (),
+				     pbFilePath, xmlFilePath);
+		
+		updatePBFilePath (context, name, pbFilePath);
+		
+		free (xmlFilePath);
+		free (pbFilePath);
+		free (name);
+		return;
+	    }
 	    else
-		addPluginFromPB (context, persistentPluginBriefPB.info (),
-				 pbFilePath, xmlFilePath);
-
-	    updatePBFilePath (context, name, pbFilePath);
-
-	    free (xmlFilePath);
-	    free (pbFilePath);
-	    free (name);
-	    return;
+	    {
+		removePB = TRUE;
+	    }
 	}
-
 	persistentPluginBriefPB.Clear ();
 	pluginInfoPBv = persistentPluginBriefPB.mutable_info ();
     }
@@ -2862,7 +2859,9 @@ loadPluginFromXMLFile (CCSContext * context, char *xmlName, char *xmlDirPath)
 #ifdef USE_PROTOBUF
     if (usingProtobuf && xmlLoaded)
     {
-	writePBFile (pbFilePath, NULL, &persistentPluginBriefPB);
+	if (removePB)
+	    remove (pbFilePath); // Attempt to remove .pb
+	writePBFile (pbFilePath, NULL, &persistentPluginBriefPB, &xmlStat);
 	updatePBFilePath (context, name, pbFilePath);
     }
 
@@ -2877,7 +2876,6 @@ static void
 loadPluginsFromXMLFiles (CCSContext * context, char *path)
 {
     struct dirent **nameList;
-    char *metadataPath;
     int nFile, i;
 
     if (!path)
@@ -3050,13 +3048,18 @@ ccsLoadPlugins (CCSContext * context)
 
 static void
 loadOptionsStringExtensionsFromXML (CCSPlugin * plugin,
-				    void * pluginPBv)
+				    void * pluginPBv,
+				    struct stat *xmlStat)
 {
     PLUGIN_PRIV (plugin);
 
     xmlDoc *doc = NULL;
     xmlNode **nodes;
     int num;
+
+    if (stat (pPrivate->xmlFile, xmlStat))
+	return;
+
     FILE *fp = fopen (pPrivate->xmlFile, "r");
     if (!fp)
 	return;
@@ -3120,14 +3123,16 @@ ccsLoadPluginSettings (CCSPlugin * plugin)
     }
 #endif
 
+    struct stat xmlStat;
+
     // Load from .xml
     if (!ignoreXML && pPrivate->xmlFile)
-	loadOptionsStringExtensionsFromXML (plugin, pluginPBToWrite);
+	loadOptionsStringExtensionsFromXML (plugin, pluginPBToWrite, &xmlStat);
 
 #ifdef USE_PROTOBUF
     if (pluginPBToWrite && pPrivate->pbFilePath && loadedAtLeastBriefPB)
 	writePBFile (pPrivate->pbFilePath, (PluginMetadata *) pluginPBToWrite,
-		     NULL);
+		     NULL, &xmlStat);
 #endif
     D (D_FULL, "done\n");
 
