@@ -1515,13 +1515,20 @@ ccsGetSortedPluginStringList (CCSContext * context)
 	ap = ccsPluginListRemove (ap, p, FALSE);
 
     int len = ccsPluginListLength (ap);
-    
+    if (len == 0)
+    {
+	ccsStringListFree (rv, TRUE);
+	return NULL;
+    }
     int i, j;
     /* TODO: conflict handling */
 
     PluginSortHelper *plugins = calloc (1, len * sizeof (PluginSortHelper));
     if (!plugins)
+    {
+	ccsStringListFree (rv, TRUE);
 	return NULL;
+    }
 
     for (i = 0, list = ap; i < len; i++, list = list->next)
     {
@@ -2636,7 +2643,10 @@ ccsImportFromFile (CCSContext *context,
 
 		    if (ccsIniGetString (importFile, plugin->name,
 					 keyName, &value))
-			ccsSetString (setting, value);
+		    {
+		    	ccsSetString (setting, value);
+		    	free (value);
+		    }
 		}
 		break;
 	    case TypeKey:
@@ -2689,7 +2699,10 @@ ccsImportFromFile (CCSContext *context,
 		    char *value;
 		    if (ccsIniGetString (importFile, plugin->name,
 					 keyName, &value))
-			ccsSetMatch (setting, value);
+		    {
+		    	ccsSetMatch (setting, value);
+		    	free (value);
+		    }
 		}
 		break;
 	    case TypeList:
